#ifndef STRUCTURES_H
#define STRUCTURES_H

#include "IGL.h"
#include "DirectoryServices.h"
#include "Globals.h"

#include <fstream.h>
using namespace std;


enum ShotBehaviour {Falling, Aimed, Homing};


struct FadingPoint {
    TypePoint position, velocity;
    double initialLife, life;
    long baseColour;
};

struct Shot {
    char *name;
    TypePolyAnimation graphic;
    SoundPtr sound;
    double cost;
    int reloadTime;
    int damage;
    double speed;
    ShotBehaviour behaviour;
};

struct Player {
    char *name;
    TypePolyAnimation graphic;
    SoundPtr dieSound;
    double speed; // = 250 * updatePeriod
    TypePoint position, destination;
    double rechargeRate, currentCharge, currentCapacity; // = updatePeriod, = 1.0, = 1.0, = 5.0
    int waitCount, currentShotType; // = 10, = 0, = 0
    int invulnerabilityTime; // = 0
    bool firing, didFire; // = false, = true
    int score, lives; // = 0, = 3
    int difficulty; // = 2
    
    vector< pair<pair<TypePoint, TypePoint>, int> > shots; // First is the shot co-ordinate & velocity (respectively), second the shot type
};

struct Enemy {
    char *name;
    TypePolyAnimation graphic;
    SoundPtr hitSound, dieSound;
    int value;
    int hitPoints;
    int numberOfShotTypes;
    int *shotTypes;
    double *shotProbabilities;
};

struct EnemyInstance {
    bool inGrid;
    TypePoint position;
    int hitPoints;
};

struct Level {
    char *name, *introText, *endText;
    SoundPtr backgroundMusic;
    int rowCount, columnCount;
    double horizontalSpacing, verticalSpacing, speed;
    Enemy *types;
    bool timeLimited, bonusModified;
    int timeLimit, bonusModifierKickIn;
    double bonusModifier;
    int bonus;
};


bool loadShot(int type, Shot &shot);
void unloadShot(Shot &shot);

bool loadLevel(int index, Level &level);
void unloadLevel(Level &level);

bool loadEnemy(const char *name, Enemy &enemy);
void unloadEnemy(Enemy &enemy);

#endif
